package Diario;

#

require EntradaSalida;
use DB;

@ISA=qw(DB);

$SI=1;
$NO=0;

sub Nuevo
{
	my $this=shift;
	my $clase=ref($this) || $this;
	my ($nombre_diario_completo,$nombre_diario_resumen)=@_;
	my $diario={};

	$diario->{Pantalla}=Nueva EntradaSalida;
	$diario->{Pantalla}->{CadenaTabulacion}=' ';
	$diario->{Completo}=Nueva EntradaSalida(">$nombre_diario_completo") if (defined $nombre_diario_completo);
	$diario->{Resumen}=Nueva EntradaSalida(">$nombre_diario_resumen") if (defined $nombre_diario_resumen);

	$diario->{Tracear}=$NO;
	$diario->{Llamadas}=[];
	$diario->{NumLlamadas}=0;
	$diario->{MinRepetidosAgrupar}=4;

	bless $diario, $clase;
	return $diario;
}


sub Mostrar
{
	my $diario=shift;
	my ($cadena)=@_;

	$diario->{Pantalla}->Escribir($cadena);
}

sub Anotar
{
	my $diario=shift;
	my ($cadena)=@_;
 
#	$diario->{Pantalla}->Escribir($cadena);
	$diario->{Completo}->Escribir($cadena) if (defined $diario->{Completo});
}

sub Chivato
{
	my $diario=shift;
	my ($cadena)=@_;

	if($diario->{Tracear})
	{
		$cadena="Chivato: $cadena";
		$diario->{Pantalla}->Escribir($cadena);
		$diario->{Completo}->Escribir($cadena) if (defined $diario->{Completo});
	}
}

sub Apuntar
{
	my $diario=shift;
	my ($cadena)=@_;

	$diario->{Pantalla}->Escribir($cadena);
	$diario->{Completo}->Escribir($cadena) if (defined $diario->{Completo});
	$diario->{Resumen}->Escribir($cadena) if (defined $diario->{Resumen});
}

sub Tabular
{
	my $diario=shift;
	my ($n)=@_;

	$diario->{Pantalla}->Tabular($n);
	$diario->{Completo}->Tabular($n);
	$diario->{Resumen}->Tabular($n);
}

sub Destabular
{
	my $diario=shift;
	my ($n)=@_;

	$diario->{Pantalla}->Destabular($n);
	$diario->{Completo}->Destabular($n);
	$diario->{Resumen}->Destabular($n);
}

sub Cerrar
{
	my $diario=shift;

	$diario->{Completo}->Cerrar  if (defined $diario->{Completo});
	$diario->{Resumen}->Cerrar  if (defined $diario->{Resumen});
}

sub Warning
{
	my $diario=shift;
	my ($cadena)=@_;

	$diario->Apuntar("Warning: $cadena\n");
}

sub Error
{
	my $diario=shift;
	my ($cadena)=@_;

	$diario->Apuntar("Error: $cadena\n");
}

sub Empilar
{
	my $diario=shift;
	my @parametros=@_;

	if($diario->{Tracear})
	{
		my ($pack,$file,$line,$llamada)=caller(1);

		$llamada=~s/::/\->/;
		$llamada="$llamada(";
		my $ultimo_parametro=pop @parametros;
		foreach my $parametro (@parametros)
		{
			$llamada="$llamada$parametro, ";
		}
		$llamada="$llamada$ultimo_parametro" if (defined $ultimo_parametro);
		$llamada="$llamada)";
		
		$diario->Apuntar("$llamada\n");
		$diario->Apuntar("{\n");
		$diario->Tabular;

		push @{$diario->{Llamadas}}, $llamada;
		$diario->{NumLlamadas}++;
	}

	return @parametros;
}

sub Desempilar
{
	my $diario=shift;

	if($diario->{Tracear})
	{
		my $llamada=pop @{$diario->{Llamadas}};
		$diario->{NumLlamadas}--;

		$diario->Destabular;
		$diario->Apuntar("}\n");
	}
}

sub MostrarPila
{
	my $diario=shift;

	foreach my $llamada (@{$diario->{Llamadas}}) 
	{
		$diario->Anotar("$llamada\n");
		$diario->Tabular;
	}
	$diario->Destabular($diario->{NumLlamadas});
}

sub OrdenarVector
{
	my $diario=shift;
	my ($vector,$frecuencias)=@_;

	my @vector_ordenado=sort { $a <=> $b } @{$vector};

	foreach my $elemento (@vector_ordenado)
	{
		if(exists $$frecuencias{$elemento})
		{
			$$frecuencias{$elemento}++;
		}
		else
		{
			$$frecuencias{$elemento}=1;
		}
	}

	return @vector_ordenado;
}

sub ListaOrdenada
{
	my $diario=shift;
	
	my @vector=@_;
	my %frecuencias=();
		
	my @vector_ordenado=$diario->OrdenarVector(\@vector,\%frecuencias);

	my $num_elementos=scalar(@vector_ordenado);
	for(my $i=0;$i<$num_elementos;$i++)
	{
		$elemento=$vector_ordenado[$i];
		if($frecuencias{$elemento}>=$diario->{MinRepetidosAgrupar})
		{
			$diario->Anotar("$elemento($frecuencias{$elemento}) ");
			$i+=($frecuencias{$elemento}-1);
		}
		else
		{
			$diario->Anotar("$elemento ");
		}
	}
	$diario->Anotar("\n");
}

sub ListaOrdenadaDescendiente
{
	my $diario=shift;
	
	my @vector=@_;
	my %frecuencias=();
		
	my @vector_ordenado=reverse $diario->OrdenarVector(\@vector,\%frecuencias);

	my $num_elementos=scalar(@vector_ordenado);
	for(my $i=0;$i<$num_elementos;$i++)
	{
		$elemento=$vector_ordenado[$i];
		if($frecuencias{$elemento}>=$diario->{MinRepetidosAgrupar})
		{
			$diario->Anotar("$elemento($frecuencias{$elemento}) ");
			$i+=($frecuencias{$elemento}-1);
		}
		else
		{
			$diario->Anotar("$elemento ");
		}
	}
	$diario->Anotar("\n");
}


1;
