package GestorDatos;

#

require Diario;
require DBI;

$SI=1;
$NO=0;

sub Nuevo
{
	my $this=shift;
	my $clase=ref($this) || $this;
	my $gestor_datos={};
	my ($nombre,$usuario,$password)=@_;

	$gestor_datos->{Conectado}=$NO;
	$gestor_datos->{SentenciaNum}=1;
	$gestor_datos->{SentenciasAbiertas}=0;
	$gestor_datos->{Sentencia}={};
	$gestor_datos->{SentenciaActiva}={};
	$gestor_datos->{Simular}=$NO;

	$gestor_datos->{Nombre}=$nombre;
	$gestor_datos->{Usuario}=$usuario;
	$gestor_datos->{Password}=$password;

	bless $gestor_datos,$clase;
	return $gestor_datos;
}

# Establece una conexion a la base de datos
sub Conectar
{
	my $gestor_datos=shift;

	if($gestor_datos->{Conectado}==$NO)
	{
		$gestor_datos->{Apuntador} = connect("dbi:ODBC:$gestor_datos->{Nombre}", "$gestor_datos->{Usuario}", "$gestor_datos->{Password}");
		if(defined $gestor_datos->{Apuntador})
		{
			$gestor_datos->{Conectado}=$SI;
		}
	}

	return $gestor_datos->{Conectado};
}

# Desactiva una conexion a la base de datos previamente establecida
sub Desconectar
{
	my $gestor_datos=shift;

	if($gestor_datos->{Conectado})
	{
		foreach my $id (keys %{$gestor_datos->{Sentencia}}) 
		{
			$gestor_datos->{Sentencia}{$id}->finish;
			$gestor_datos->{SentenciasAbiertas}--;
		}
		$gestor_datos->{Apuntador}->disconnect;
		$gestor_datos->{Conectado}=$NO;
	}
}

sub SentenciaActivada
{
	my $gestor_datos=shift;
	my ($id)=@_;

	if(exists $gestor_datos->{SentenciaActiva}{$id})
	{
		return $SI;
	}
	return $NO;
}

sub PrepararSentencia
{
	my $gestor_datos=shift;
	my ($sentencia)=@_;

	my $id=$gestor_datos->{SentenciaNum}++;
	$gestor_datos->{Sentencia}{$id}=$gestor_datos->{Apuntador}->prepare($sentencia);
	if(defined $gestor_datos->{Sentencia}{$id})
	{
		$gestor_datos->{SentenciaActiva}{$id}=$SI;
		$gestor_datos->{SentenciasAbiertas}++;
		return $id;
	}
	else
	{
		return 0;
	}
}

sub EjecutarSentenciaParametrizada
{
	my $gestor_datos=shift;
	my $id=shift;
	my $sentencia=shift;
	if($gestor_datos->SentenciaActivada($$id)==$NO)
	{
		$$id=$gestor_datos->PrepararSentencia($sentencia);
	}
	if($gestor_datos->SentenciaActivada($$id)==$SI)
	{
#print "ESP: $$id  $sentencia : @_\n";
		if(defined $gestor_datos->{Sentencia}{$$id}->execute(@_))
		{
			return $SI;
		}
		else
		{
			return $NO;
		}
	}	
}

sub EjecutarSentencia
{
	my $gestor_datos=shift;
	my ($id,$sentencia)=@_;

	if($gestor_datos->{Conectado})
	{
		$$id=$gestor_datos->PrepararSentencia($sentencia);
		if($gestor_datos->SentenciaActivada($$id))
		{
			if(defined $gestor_datos->{Sentencia}{$$id}->execute)
			{
				return $SI;
			}
			else
			{
				return $NO;
			}
		}
	}
	return $NO;
}

sub FinalizarSentencia
{
	my $gestor_datos=shift;
	my ($id)=@_;

	if($gestor_datos->{Conectado})
	{
		if($gestor_datos->SentenciaActivada($id))
		{
			$gestor_datos->{Sentencia}{$id}->finish;
			delete $gestor_datos->{Sentencia}{$id};
			delete $gestor_datos->{SentenciaActiva}{$id};
			$gestor_datos->{SentenciasAbiertas}--;
		}
	}
}

sub LeerCursor
{
	my $gestor_datos=shift;
	my ($sentencia,$funcion)=@_;

	if($gestor_datos->{Conectado})
	{
		my $id=0;
		if($gestor_datos->EjecutarSentencia(\$id,$sentencia))
		{
			while((my @registro = $gestor_datos->{Sentencia}{$id}->fetchrow_array) >0)
			{
				&$funcion(@registro);
			}
		}
		$gestor_datos->FinalizarSentencia($id);
	}
}

sub Confirmar
{
	my $gestor_datos=shift;

	if($gestor_datos->{Conectado})
	{
		$gestor_datos->{Apuntador}->commit;
	}
}

sub Deshacer
{
	my $gestor_datos=shift;

	if($gestor_datos->{Conectado})
	{
		$gestor_datos->{Apuntador}->rollback;
	}
}

# Hace una consulta y devuelve la primera fila
sub Consultar
{
	my $gestor_datos=shift;
	my $sentencia=shift;

	if($gestor_datos->{Conectado})
	{
		my $id=0;
		if($gestor_datos->EjecutarSentenciaParametrizada(\$id,$sentencia,@_))
		{
			my @resultado=$gestor_datos->{Sentencia}{$id}->fetchrow_array;
#			$r=$resultado[0];
#			print "RES[1]=$r\n";
			$gestor_datos->FinalizarSentencia($id);
			return @resultado;
		}
		$gestor_datos->FinalizarSentencia($id);
	}
}

sub SimularSentencia
{
	my $gestor_datos=shift;
	my ($id,$sentencia)=@_;

	if(($gestor_datos->{Conectado})&&(defined $gestor_datos->{Diario}))
	{
		$gestor_datos->{Diario}->Anotar("\n$sentencia\n");
	}

	return $SI;
}

sub SimularSentenciaParametrizada
{
	my $gestor_datos=shift;
	my $id=shift;
	my $sentencia=shift;

	if(($gestor_datos->{Conectado})&&(defined $gestor_datos->{Diario}))
	{
		my @paramatros=@_;
		my $num_params=scalar @paramatros;
		if($num_params>0)
		{
			$gestor_datos->{Diario}->Anotar("\n$sentencia (");
			my $ultimo=$num_params-1;
			for(my $i=0;$i<$num_params-1;$i++)
			{
				my $param=$paramatros[$i];
				if(!(defined $param))
				{
					$param="NULL, ";
				}
				if($param=~/\A[\d\-\+\.eE]+\Z/)
				{
					$gestor_datos->{Diario}->Anotar("$param, ");
				}
				else
				{
					$gestor_datos->{Diario}->Anotar("\'$param\', ");
				}
			}
			my $p=$paramatros[$ultimo];
			if($p=~/\A[\d\-\+\.eE]+\Z/)
			{
				$gestor_datos->{Diario}->Anotar("$p");
			}
			else
			{
				$gestor_datos->{Diario}->Anotar("\'$p\'");
			}
			$gestor_datos->{Diario}->Anotar(")\n");
		}
		else
		{
			$gestor_datos->{Diario}->Anotar("\n$sentencia\n");
		}
	}	

	return $SI;
}

sub ErrorSentencia
{
	my $gestor_datos=shift;
	my ($id,$sentencia)=@_;

	if(($gestor_datos->{Conectado})&&(defined $gestor_datos->{Diario}))
	{
		$gestor_datos->{Diario}->Anotar("\nError al ejecutar $sentencia\n");
	}
}

sub ErrorSentenciaParametrizada
{
	my $gestor_datos=shift;
	my $id=shift;
	my $sentencia=shift;

	if(($gestor_datos->{Conectado})&&(defined $gestor_datos->{Diario}))
	{
		$gestor_datos->{Diario}->Anotar("\nError al ejecutar $sentencia (");
		foreach my $param (@_)
		{
			if(!(defined $param))
			{
				$param="NULL, ";
			}
			$gestor_datos->{Diario}->Anotar("$param, ");
		}
		$gestor_datos->{Diario}->Anotar(")\n");
	}	
}	

sub Ejecutar
{
	my $gestor_datos=shift;
	my $id=shift;
	my $sentencia=shift;
	my @parametros=@_;

	my $ok=$NO;
    if($gestor_datos->{Simular})
	{
		$ok=$gestor_datos->SimularSentenciaParametrizada($id,$sentencia,@parametros);
	}
	else
	{
#		$gestor_datos->{Diario}->Anotar("Ejecutando: $sentencia");
		$ok=$gestor_datos->EjecutarSentenciaParametrizada($id,$sentencia,@parametros);
	}
	if($ok)
	{
		$gestor_datos->{Diario}->Anotar(".");
#		$gestor_datos->{Diario}->Anotar(". OK\n");
	}
	else
	{
		$gestor_datos->{Diario}->Anotar("\n");
		$gestor_datos->ErrorSentenciaParametrizada($id,$sentencia,@parametros);
	}

	$gestor_datos->FinalizarSentencia($$id);
	return $ok;
}

sub EjecutarSentencias
{
	my $gestor_datos=shift;
	my $id=shift;
	my @sentencias=@_;

	my $num_sentencias=scalar(@sentencias);
	my $actual=0;
	my $ok=$SI;
	while(($actual<$num_sentencias)&&($ok==$SI))
	{
		$ok=$gestor_datos->Ejecutar($id,$sentencias[$actual]);
		$actual++;
	}
	return $ok;
}

1;
