package Cola;

# Implementa una cola de prioridad mediante una estructura heap para elementos 
# {clave,valor} ordenando por valor.

require Util;
require Diario;

use Global;

$CLAVE=0;
$VALOR=1;

sub Nueva
{
	my $this=shift;
	my $clase=ref($this) || $this;
	my ($tipo)=@_;
	my $cola={};

	$cola->{Heap}=[];
	$cola->{Heap}[0]=[0,0];
	$cola->{Posicion}={};
	$cola->{NumElementos}=0;

	if(defined $tipo)
	{
		$cola->{Tipo}=$tipo;
	}
	else
	{
		$cola->{Tipo}=$MINIMOS;
	}

	bless $cola,$clase;
	return $cola;
}

# O(n)
sub Construir
{
	my $cola=shift;
	my ($claves,$valores)=@_;

	my $num_valores=scalar(@$claves);
#$diario->Anotar("Construyendo heap con $num_valores valores\n");
	for(my $i=1;$i<=$num_valores;$i++)
	{
#$diario->Anotar("Insertando clave $$claves[$i-1], valor $$valores[$i-1] en $i\n");
		$cola->{Heap}[$i][$CLAVE]=$$claves[$i-1];
		$cola->{Heap}[$i][$VALOR]=$$valores[$i-1];
		$cola->{Posicion}->{$$claves[$i-1]}=$i
	}

	$cola->{NumElementos}=$num_valores;
	for(my $i=int($cola->{NumElementos}/2);$i>=1;$i--)
	{
		$cola->BajarElemento($i,$cola->{Heap}[$i][$CLAVE],$cola->{Heap}[$i][$VALOR]);
	}
}

sub Cola::Padre
{
	my ($i)=@_;
	return Util::Truncar($i/2);
}

sub Cola::HijoDerecho
{
	my ($i)=@_;
	return $i*2+1;
}

sub Cola::HijoIzquierdo
{
	my ($i)=@_;
	return $i*2;
}

sub Mejor
{
	my $cola=shift;
	my ($valor,$posicion)=@_;

	if($cola->{Tipo}==$MINIMOS)
	{
		return ($cola->{Heap}[$posicion][$VALOR]>$valor);
	}
	return ($cola->{Heap}[$posicion][$VALOR]<$valor);
}

sub Peor
{
	my $cola=shift;
	my ($valor,$posicion)=@_;

	if($cola->{Tipo}==$MINIMOS)
	{
		return ($cola->{Heap}[$posicion][$VALOR]<$valor);
	}
	return ($cola->{Heap}[$posicion][$VALOR]>$valor);
}

# Retornamos la posicion llegada.
sub SubirElemento
{
	my $cola=shift;
	my ($desde,$clave,$valor)=@_;

#	$diario->Anotar "Subiendo desde $desde ($clave,$valor)\n";
	$aaa=Cola::Padre($desde);
#	$diario->Anotar "Padre(i)=$aaa      $cola->{Heap}[$aaa][$VALOR]\n";

	$i=$desde;
	while(($i>1)&&($cola->Mejor($valor,Cola::Padre($i))))
	{
#		$diario->Anotar "pasando por $i\n";
		$cola->{Heap}[$i]=$cola->{Heap}[Cola::Padre($i)];
		$cola->{Posicion}->{$cola->{Heap}[$i][$CLAVE]}=$i;
		$i=Cola::Padre($i);
	}
	$cola->{Heap}[$i]=[$clave,$valor];
	$cola->{Posicion}->{$clave}=$i;

	return $i;
}

sub BajarElemento
{
	my $cola=shift;
	my ($desde,$clave,$valor)=@_;

	my ($i,$d,$min);
	
#	$diario->Anotar("Bajando desde $desde ($clave,$valor)\n");

	$i=Cola::HijoIzquierdo($desde);
	$d=Cola::HijoDerecho($desde);

#	$diario->Anotar("Hijoizq=$i Hijoder=$d\n");
	if(($i<=$cola->{NumElementos})&&($cola->Peor($valor,$i)))
	{
		$min=$i;
#		$diario->Anotar("El de la izquierda es el minimo\n");
	}
	else
	{
		$min=$desde;
	}
	if(($d<=$cola->{NumElementos})&&($cola->Peor($cola->{Heap}[$min][$VALOR],$d)))
	{
#		$diario->Anotar("El de la derecha es el minimo\n");
		$min=$d;
	}
	if($min!=$desde)
	{
#		$diario->Anotar("Modificando posicin.\n");
		$cola->{Heap}[$desde]=$cola->{Heap}[$min];
#		$cola->{Heap}[$desde][$CLAVE]=$cola->{Heap}[$min][$CLAVE];
#		$cola->{Heap}[$desde][$VALOR]=$cola->{Heap}[$min][$VALOR];
		$cola->{Heap}[$min]=[$clave,$valor];
		$cola->{Posicion}->{$clave}=$min;
		$cola->{Posicion}->{$cola->{Heap}[$desde][$CLAVE]}=$desde;
		$cola->BajarElemento($min,$clave,$valor);
	}
}

# Retorna si se ha modificado la posicion de cabeza.
sub Modificar
{
	my $cola=shift;
	my ($clave,$valor)=@_;

#$diario->Anotar("Modificando $clave, $valor\n");
	if(exists $cola->{Posicion}->{$clave})
	{
		my $posicion=$cola->{Posicion}->{$clave};
#$diario->Anotar("		Est en la posicion $posicion\n");
		if($cola->Mejor($valor,$posicion))
		{
#$diario->Anotar("		Mejora, subimos elemento\n");
			$cola->{Heap}[$posicion][$VALOR]=$valor;
			$posicion=$cola->SubirElemento($posicion,$clave,$valor) if ($posicion!=1);
		}
		else
		{
			$cola->{Heap}[$posicion][$VALOR]=$valor;
#$diario->Anotar("		No mejora, bajamos elemento $cola->{Heap}[$posicion][$VALOR]\n");
			$cola->BajarElemento($posicion,$clave,$valor) if ($posicion!=$cola->{NumElementos});
#$diario->Anotar("		Fin bajando elemento $cola->{Heap}[$posicion][$VALOR]. NUMELEMS: $cola->{NumElementos}\n");
		}

		return ($posicion==1);
	}

	return $NO;
}

# Retorna si se ha modificado la posicion de cabeza.
sub Incrementar
{
	my $cola=shift;
	my ($clave,$valor)=@_;

#$diario->Anotar("Modificando $clave, $valor\n");
	if(exists $cola->{Posicion}->{$clave})
	{
		my $posicion=$cola->{Posicion}->{$clave};
#$diario->Anotar("		Est en la posicion $posicion\n");
		if($cola->Mejor($valor,$posicion))
		{
#$diario->Anotar("		Mejora, subimos elemento\n");
			$cola->{Heap}[$posicion][$VALOR]+=$valor;
			$posicion=$cola->SubirElemento($posicion,$clave,$cola->{Heap}[$posicion][$VALOR]) if ($posicion!=1);
		}
		else
		{
			$cola->{Heap}[$posicion][$VALOR]+=$valor;
#$diario->Anotar("		No mejora, bajamos elemento $cola->{Heap}[$posicion][$VALOR]\n");
			$cola->BajarElemento($posicion,$clave,$cola->{Heap}[$posicion][$VALOR]) if ($posicion!=$cola->{NumElementos});
#$diario->Anotar("		Fin bajando elemento $cola->{Heap}[$posicion][$VALOR]. NUMELEMS: $cola->{NumElementos}\n");
		}

		return ($posicion==1);
	}
	else
	{
		$cola->Insertar($clave,$valor);
	}

	return $NO;
}

# Retorna si se ha insertado la posicion de cabeza.
sub Insertar
{
	my $cola=shift;
	my ($clave,$valor)=@_;

#	$diario->Anotar("Insertando ($clave,$valor)\n");
#	if((exists $cola->{Posicion}->{$clave})&&($cola->{NumElementos}>0))
	if((exists $cola->{Posicion}->{$clave})&&($cola->{Posicion}->{$clave}<=$cola->{NumElementos})&&($cola->{Posicion}->{$clave}>0))
	{
		return $cola->Modificar($clave,$valor);
	}
	else
	{
		return $cola->SubirElemento(++$cola->{NumElementos},$clave,$valor);
	}
}

sub ConsultarMejor
{
	my $cola=shift;

	if($cola->{NumElementos}>0)
	{
		return ($cola->{Heap}[1][$CLAVE],$cola->{Heap}[1][$VALOR]);
	}
}

sub ConsultarMejoresRecursivo
{
	my $cola=shift;
	my ($desde)=@_;

$diario->Anotar("Consultando mejores desde $desde\n");
	if(($desde<=$cola->{NumElementos})&&($cola->{Heap}[$desde][$VALOR]==$cola->{Heap}[1][$VALOR]))
	{
		my @resultados=($cola->{Heap}[$desde][$CLAVE],$cola->{Heap}[$desde][$VALOR]);

		push @resultados, $cola->ConsultarMejoresRecursivo(Cola::HijoIzquierdo($desde));
		push @resultados, $cola->ConsultarMejoresRecursivo(Cola::HijoDerecho($desde));

		return @resultados;
	}

	return ();
}

sub Consultar
{
	my $cola=shift;
	my ($num)=@_;

	if(($num>=1)&&($num<=$cola->{NumElementos}))
	{
		return ($cola->{Heap}[$num][$CLAVE],$cola->{Heap}[$num][$VALOR]);
	}
}

sub ConsultarMejores
{
	my $cola=shift;

	my @m=$cola->ConsultarMejoresRecursivo(1);
$diario->Anotar("MEJORES: @m\n");
	return @m;
}

sub ConsultarAleatorio
{
	my $cola=shift;

	my $elemento=int(rand($cola->{NumElementos}))+1;

	return ($cola->{Heap}[$elemento][$CLAVE],$cola->{Heap}[$elemento][$VALOR]);
}



sub ExtraerMejor
{
	my $cola=shift;

	my ($clave,$valor);

	if($cola->{NumElementos}>0)
	{
		$clave=$cola->{Heap}[1][$CLAVE];
		$valor=$cola->{Heap}[1][$VALOR];
		delete $cola->{Posicion}->{$clave};
		#		$diario->Anotar "El minimo es $cola->{Heap}[1]\n";
		if($cola->{NumElementos}>1)
		{
			$cola->{Heap}[1]=$cola->{Heap}[$cola->{NumElementos}];
			$cola->{Posicion}->{$cola->{Heap}[1][$CLAVE]}=1;
			$cola->{NumElementos}--;
			$cola->BajarElemento(1,$cola->{Heap}[1][$CLAVE],$cola->{Heap}[1][$VALOR]);
		}
		else
		{
			$cola->{NumElementos}--;
		}
		return ($clave,$valor);
	}
}

# Extrae todos los elementos de la cola de prioridad que sn iguales en valor que el mejor.
sub ExtraerMejores
{
	my $cola=shift;

	if($cola->{NumElementos}>0)
	{
		my ($clave,$valor)=$cola->ExtraerMejor;
		my @resultados=($clave,$valor);

		my ($siguiente_clave,$siguiente_valor)=$cola->ConsultarMejor;
		while($siguiente_valor==$valor)
		{
			($clave,$valor)=$cola->ExtraerMejor;
			push @resultados, $clave, $valor;
			($siguiente_clave,$siguiente_valor)=$cola->ConsultarMejor;
		}
		return @resultados;
	}
	return undef;
}

# Extrae el elemento corresponiente a la clave de la cola.
sub Extraer
{
	my $cola=shift;
	my ($clave)=@_;

	if(($cola->{NumElementos}>0)&&(exists $cola->{Posicion}->{$clave}))
	{
		my $posicion=$cola->{Posicion}->{$clave};
#$diario->Anotar("Extrayendo de la posicion $posicion el elemento ($cola->{Heap}[$posicion][$CLAVE], $cola->{Heap}[$posicion][$VALOR], $cola->{Posicion}->{$cola->{Heap}[$posicion][$CLAVE]})\n");
		delete $cola->{Posicion}->{$clave};
		if($cola->{NumElementos}>1)
		{
			$cola->{Heap}[$posicion]=$cola->{Heap}[$cola->{NumElementos}];
			$cola->{Posicion}->{$cola->{Heap}[$posicion][$CLAVE]}=$posicion;
			$cola->{NumElementos}--;
			$cola->BajarElemento($posicion,$cola->{Heap}[$posicion][$CLAVE],$cola->{Heap}[$posicion][$VALOR]);
		}
		else
		{
			$cola->{NumElementos}--;
		}
		return ($posicion==1);
	}

	return $NO;
}

sub Vacia
{
	my $cola=shift;

	return($cola->{NumElementos}==0);
}

sub Mostrar
{
	my $cola=shift;

	for($i=1;$i<$cola->{NumElementos};$i++)
	{
		$diario->Anotar("($cola->{Heap}[$i][$CLAVE], $cola->{Heap}[$i][$VALOR], $cola->{Posicion}->{$cola->{Heap}[$i][$CLAVE]}), ");
	}
	$diario->Anotar("($cola->{Heap}[$i][$CLAVE], $cola->{Heap}[$i][$VALOR], $cola->{Posicion}->{$cola->{Heap}[$i][$CLAVE]})") if ($i==$cola->{NumElementos});
}

1;
